// bi_Color:   Driver, ISR & other support for 2-pin bi-Color LEDs
// July 2024 Ron Miller (see RonM9 on Instructables.com) 

// ============================================================= implement a 10 KHz (100us) timer/counter
#include "pico/time.h"

volatile uint32_t counter = 0; // Stores the interrupt count
unsigned int msCnt=0;   // pseudo MSecs, incremented inside of sub-millisec ISR
unsigned int mySecs=0;
struct repeating_timer timer;  // Initialize the repeating timer structure

// ================== LED Hardware Configuration for WaveShare RP2040-Zero breadboard setup   
byte nleds=8;
const int lites[] = {1, 2, 3, 4, 5, 6, 7, 8};     //the LED pins
#define LED_VCC 0    // D0 supplies +V to the 8 LEDs
// ==================

// ----- assign colors to the LEDs
const char stdClrs[] = {0,1,1,4,4,8,8,3,3};  // r,y,g,o
const char RedClr[] =  {0,1,1,1,1,1,1,1,1};  // all Red
const char YelClr[] =  {0,4,4,4,4,4,4,4,4};  // all Yel
const char GrnClr[] =  {0,8,8,8,8,8,8,8,8};  // all Green
const char OrgClr[] =  {0,3,3,3,3,3,3,3,3};  // all Orange
const char RYGClr[] =  {0,1,2,3,4,5,6,7,8};  // all R-Y-G

volatile char* LED_Color = (char*) &stdClrs;

byte RG_lvls[2][9] = {  // RG mix to produce colors   1:Red  amber  3:orange  -  5:Yellow  lite Grn   8:Green 
                          {0, 0, 1, 1, 2, 2, 3, 3, 4},   // green levels
                          {0, 5, 5, 4, 4, 3, 3, 2, 0}};    // red levels
byte LED_Lvl[9];  // holds the brightness for each LED (0-16)

//  ==================== 
void biColorDriver() {
  bool RedTime, rgOn, onLED;
  byte ledPin, led, tSlot;
  const int RG_toggle = LED_VCC;

  for (int led=1; led<=nleds; led++) {
    pinMode(lites[led-1], INPUT);  // set LED pins to high-Z during transition
  }

  //RedTime = counter&1;  // R/G divide up time equally, accross 10 time slices
  //tSlot = (counter/2)%5;
  // next two lines cause the Red LED to get twice the drive time as the Green (10:5). Done due to stronger green output.
  RedTime = (counter%3)>0;
  tSlot = (counter%15)/3;

  pinMode(RG_toggle, OUTPUT);
  digitalWrite(RG_toggle, RedTime);  // split duty cycle for R/G drive
  // -------------- drive the on Leds for current color time slice
  for (led=1; led<=nleds; led++) {
    ledPin = lites[led-1];
    onLED = LED_Lvl[led]>(msCnt%16);  // LED brightness is 0-15 (by duty cycle over 16 msecs)
    rgOn = onLED && ((RG_lvls[RedTime][LED_Color[led]] > tSlot) ); // given the color balance level & time slice should we energize this LED
    if (rgOn) {    // pin for this LED is an output only when it needs to conduct current
      pinMode(ledPin, OUTPUT);
      digitalWrite(ledPin, !RedTime);  // LED leads must be opposite polarity to energize
    }
  }
}

// ==================== 100us repeating ISR (now every 67us)
bool uSecs_ISR(struct repeating_timer *t) {
  counter++;
  //if (counter%10==0) { // 10 time slices / 1 msec 
  // now every 1 msec Disp getting 15 color updates / msec (to support R:G 2:1 ratio)
  if (counter%15==0) {
    msCnt++;  // inc my millisec counter
    if (msCnt%1000==0) mySecs++;
  }
  biColorDriver();
  return true;
}

//  ==================== 
void init_usTimer_ISR() {
    // Add a repeating timer that triggers every 100us (100 microseconds)
    //add_repeating_timer_us(-100,  timer_100us_IRS, NULL, &timer);  // for 10KHz
    add_repeating_timer_us(-67,  uSecs_ISR, NULL, &timer);  // for 15KHz
    pinMode(9, OUTPUT);
}
// ==============================================================================================================
